/*
 * julian.c:
 * Convert to/from Julian dates (in the sense that seismologists use the term).
 *
 * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
 *
 */

static const char rcsid[] = "$Id: julian.c,v 1.1 2001/01/16 12:13:39 chris Exp $";

#include <stdio.h>

int mlen[] = {-1, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

int leapyear(const int y) {
    return ((y % 4) == 0 && (y % 100) != 0) || (y % 400) == 0;
}

int monthlength(const int y, const int m) {
    if (m != 2) return mlen[m];
    else return leapyear(y) ? 29 : 28;
}

int datetojd(const int y, const int m, const int d) {
    int i, j = 0;
    for (i = 1; i < m; ++i) {
        j += mlen[i];
        if (i == 2 && leapyear(y)) ++j;
    }
    j += d;
    return j;
}

int jdtodate(const int y, int j) {
    int m = 1;
    while (j > monthlength(y, m)) {
        j -= monthlength(y, m);
        ++m;
    }
    return y * 10000 + m * 100 + j;
}

int main(int argc, char **argv) {
    if (!argv[1]) {
        
    } else {
        int i;
        i = atoi(argv[1]);
        if (!i) {
            fprintf(stderr, "julian: bogus date %s\n", argv[1]);
            return 1;
        } else if (strlen(argv[1]) == 7) {
            /* yyyyjjj */
            printf("%d\n", jdtodate(i / 1000, i % 1000));
            return 0;
        } else if (strlen(argv[1]) == 8) {
            /* yyyymmdd */
            printf("%04d%03d\n", i / 10000, datetojd(i / 10000, (i % 10000) / 100, i % 100));
            return 0;
        }
    }
}
